import json
import requests
import base64


class API:
    def __init__(self, at_base_url, at_integration_code, at_username, at_secret):
        self.base_url = at_base_url
        self.headers = {
            "ApiIntegrationCode": at_integration_code,
            "UserName": at_username,
            "Secret": at_secret,
            "Content-Type": "application/json",
        }

    def __exception_parser(self, e):
        if isinstance(e, requests.exceptions.InvalidHeader):
            return requests.exceptions.InvalidHeader(
                "Header is improperly formatted. Ensure values are strings."
            )
        if isinstance(e, requests.exceptions.MissingSchema):
            return requests.exceptions.MissingSchema("Missing / Invalid Autotask URL.")

        if isinstance(e, requests.exceptions.HTTPError) and "404 Client Error" in repr(
            e
        ):
            return requests.exceptions.HTTPError(f"Bad Autotask URL: {e}")
        if isinstance(e, requests.exceptions.HTTPError) and "Unauthorized" in repr(e):
            return requests.exceptions.HTTPError("Invalid Autotask credentials.")
        if isinstance(
            e, requests.exceptions.HTTPError
        ) and "Internal Server Error" in repr(e):
            return requests.exceptions.HTTPError(
                f"Invalid Autotask integration code OR bad query parameters."
            )

    def query_tickets(self, ticket_id):
        if not ticket_id:
            raise ValueError("You must enter a ticket id.")
        try:
            qs = f'/Tickets/query?search={{"filter":[{{"op":"eq","field":"Id","value":"{ticket_id}"}}]}}'
            resp = requests.get(self.base_url + qs, headers=self.headers)
            resp.raise_for_status()
            return resp
        except Exception as e:
            raise self.__exception_parser(e) from None

    def create_ticket(self, title, description, status, priority, company_id, queue_id):
        data = {
            "Title": title,
            "Description": description,
            "Status": status,
            "Priority": priority,
            "CompanyID": company_id,
            "QueueID": queue_id,
        }

        payload = json.dumps(data)
        try:
            resp = requests.post(
                self.base_url + "/Tickets", headers=self.headers, data=payload
            )
            resp.raise_for_status()
            ticket_id = resp.json()["itemId"]
            return ticket_id

        except Exception as e:
            raise self.__exception_parser(e) from None

    def add_ticket_attachment(
        self, ticket_id, attachment_heading, file_name, b64_encoded_attachment
    ):
        """
        Attachment needs to be base64 encoded:
        # import base64
        # data = open("sample.txt", "r").read()
        # encoded = base64.b64encode(data)
        # add_ticket_attachment(self, ticket_id, encoded)
        """
        try:
            attachment = b64_encoded_attachment.decode("utf-8")
            payload = json.dumps(
                {
                    "attachmentType": "FILE_ATTACHMENT",
                    "publish": "1",
                    "fullPath": file_name,
                    "title": attachment_heading,
                    "data": attachment,
                }
            )
        except Exception as e:
            raise e

        try:
            resp = requests.post(
                self.base_url + f"/Tickets/{ticket_id}/Attachments",
                headers=self.headers,
                data=payload,
            )
            resp.raise_for_status()

        except Exception as e:
            raise e

        return resp


import os
import base64
import unittest
import requests

from atrest.client import Client

AT_BASE_URL = os.getenv("AT_BASE_URL")
AT_INTEGRATION_CODE = os.getenv("AT_INTEGRATION_CODE")
AT_USERNAME = os.getenv("AT_USERNAME")
AT_SECRET = os.getenv("AT_SECRET")

TEST_QUEUE_ID = 29683487


class TestClientInitialization(unittest.TestCase):
    def testSuccessfulClientInitialization(self):
        client = Client("test", "test", "test", "test")
        self.assertIsInstance(client, Client)


class TestWithBadSecrets(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.client = Client(
            AT_BASE_URL,
            AT_INTEGRATION_CODE,
            AT_USERNAME,
            AT_SECRET,
        )

    def testSuccessfulTicketSubmission(self):
        self.assertIsInstance(self.client, Client)
        ticket_id = self.client.create_ticket("test", "test", 1, 1, 0, TEST_QUEUE_ID)
        self.assertIsInstance(ticket_id, int)

    def testMalformedHeader(self):
        self.client.api.headers["ApiIntegrationCode"] = 111111111
        with self.assertRaisesRegex(
            requests.exceptions.InvalidHeader, "Header is improperly formatted."
        ):
            self.client.create_ticket("test", "test", 1, 1, 0, TEST_QUEUE_ID)

    def testInvalidBaseUrl(self):
        self.client.api.base_url = "test"
        with self.assertRaisesRegex(
            requests.exceptions.MissingSchema, "Missing / Invalid Autotask URL."
        ):
            self.client.create_ticket("test", "test", 1, 1, 0, TEST_QUEUE_ID)

    def testMalformedAutotaskUrl(self):
        self.client.api.base_url = (
            "https://webservices15.autotask.net/atservicesrest/v1.05"
        )
        with self.assertRaisesRegex(requests.exceptions.HTTPError, "Bad Autotask URL"):
            self.client.create_ticket("test", "test", 1, 1, 0, TEST_QUEUE_ID)

    def testInvalidIntegrationCode(self):
        self.client.api.headers["ApiIntegrationCode"] = "11111111111111"
        with self.assertRaisesRegex(
            requests.exceptions.HTTPError, "Invalid Autotask integration code"
        ):
            self.client.create_ticket("test", "test", 1, 1, 0, TEST_QUEUE_ID)

    def testInvalidUsername(self):
        self.client.api.headers["UserName"] = "fakeemail@strivehealth.com"
        with self.assertRaisesRegex(
            requests.exceptions.HTTPError, "Invalid Autotask credentials."
        ):
            self.client.create_ticket("test", "test", 1, 1, 0, TEST_QUEUE_ID)

    def testInvalidSecret(self):
        self.client.api.headers["Secret"] = "reallyFakePassword"
        with self.assertRaisesRegex(
            requests.exceptions.HTTPError, "Invalid Autotask credentials."
        ):
            self.client.create_ticket("test", "test", 1, 1, 0, TEST_QUEUE_ID)


class TestCreateTicket(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.client = Client(
            AT_BASE_URL,
            AT_INTEGRATION_CODE,
            AT_USERNAME,
            AT_SECRET,
        )

    def testBadStatus(self):
        BAD_STATUS = 1111111111111
        with self.assertRaisesRegex(
            requests.exceptions.HTTPError, "bad query parameters"
        ):
            ticket_id = self.client.create_ticket(
                "test", "test", BAD_STATUS, 1, 0, TEST_QUEUE_ID
            )

    def testBadPriority(self):
        BAD_PRIORITY = 1111111111111
        with self.assertRaisesRegex(
            requests.exceptions.HTTPError, "bad query parameters"
        ):
            ticket_id = self.client.create_ticket(
                "test", "test", 1, BAD_PRIORITY, 0, TEST_QUEUE_ID
            )

    def testBadCompanyId(self):
        BAD_COMPANY_ID = 1111111111111
        with self.assertRaisesRegex(
            requests.exceptions.HTTPError, "bad query parameters"
        ):
            ticket_id = self.client.create_ticket(
                "test", "test", 1, 1, BAD_COMPANY_ID, TEST_QUEUE_ID
            )

    def testBadQueueId(self):
        BAD_QUEUE_ID = 1111111111111
        with self.assertRaisesRegex(
            requests.exceptions.HTTPError, "bad query parameters"
        ):
            ticket_id = self.client.create_ticket("test", "test", 1, 1, 0, BAD_QUEUE_ID)


class TestQueryTickets(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.client = Client(
            AT_BASE_URL,
            AT_INTEGRATION_CODE,
            AT_USERNAME,
            AT_SECRET,
        )

    def testGoodQuery(self):
        resp = self.client.query_tickets("9877")
        self.assertIsNotNone(resp)
        self.assertIsInstance(resp, requests.Response)

    def testBadQuery(self):
        with self.assertRaisesRegex(ValueError, "You must enter a ticket id"):
            self.client.query_tickets("")


class TestCreateAttachment(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.client = Client(
            AT_BASE_URL,
            AT_INTEGRATION_CODE,
            AT_USERNAME,
            AT_SECRET,
        )

        self.ticket_number = "10505"
        self.attachment_heading = "test-report-heading"
        self.file_name = "test-report.csv"
        self.encoded = base64.b64encode(
            "header, header, header, header\ntest, test, test, test".encode("utf-8")
        )

    def testGoodTicketAttachment(self):
        resp = self.client.add_ticket_attachment(
            self.ticket_number, self.attachment_heading, self.file_name, self.encoded
        )
        self.assertIsNotNone(resp)
        self.assertIsInstance(resp, requests.Response)

    def testBadTicketNumber(self):
        with self.assertRaisesRegex(Exception, "500 Server Error"):
            resp = self.client.add_ticket_attachment(
                "1",
                self.attachment_heading,
                self.file_name,
                self.encoded,
            )


class TestCreateTicketWithAttachment(unittest.TestCase):
    @classmethod
    def setUp(self):
        self.client = Client(
            AT_BASE_URL,
            AT_INTEGRATION_CODE,
            AT_USERNAME,
            AT_SECRET,
        )

        self.title = "Test Ticket With Attachment"
        self.description = "test ticket description"
        self.status = 1
        self.priority = 1
        self.company_id = 0
        self.queue_id = TEST_QUEUE_ID
        self.attachment_heading = "test-report-heading"
        self.file_name = "test-report.csv"
        self.encoded = base64.b64encode(
            "header, header, header, header\ntest, test, test, test".encode("utf-8")
        )

    def testGoodTicketAttachment(self):
        resp = self.client.create_ticket_with_attachment(
            self.title,
            self.description,
            self.status,
            self.priority,
            self.company_id,
            self.queue_id,
            self.attachment_heading,
            self.file_name,
            self.encoded,
        )
        self.assertIsNotNone(resp)
        self.assertIsInstance(resp, int)



from setuptools import setup, find_packages

from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README-pypi.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="atrest",
    version="1.0.2",
    author="Jonathan Weaver",
    author_email="createchange@protonmail.com",
    long_description=long_description,
    long_description_content_type="text/markdown",
    description="Basic Autotask REST Client",
    packages=find_packages(exclude=["test"]),
    install_requires=[
        "requests",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
)



import requests
from atrest.api import API


class Client:
    def __init__(self, at_base_url, at_integration_code, at_username, at_secret):
        self.api = API(at_base_url, at_integration_code, at_username, at_secret)

    def query_tickets(self, ticket_id) -> requests.Response:
        """
        Query for a ticket by ticket id.
        Returns either the ticket information in json, or raises an exception.
        """
        return self.api.query_tickets(ticket_id)

    def create_ticket(
        self, title, description, status, priority, company_id, queue_id
    ) -> int:
        """
        Create a ticket.
        Returns either the ticket ID of the created ticket, or raises an exception.
        """
        return self.api.create_ticket(
            title, description, status, priority, company_id, queue_id
        )

    def add_ticket_attachment(
        self, ticket_id, attachment_heading, file_name, b64_encoded_attachment
    ) -> int:
        """
        Create a ticket.
        Returns either the ticket ID of the created ticket, or raises an exception.

        Attachment needs to be base64 encoded:
        # import base64
        # data = open("sample.txt", "r").read()
        # encoded = base64.b64encode(data.encode('utf-8'))
        # add_ticket_attachment(self, ticket_id, encoded)
        """
        return self.api.add_ticket_attachment(
            ticket_id, attachment_heading, file_name, b64_encoded_attachment
        )

    def create_ticket_with_attachment(
        self,
        title,
        description,
        status,
        priority,
        company_id,
        queue_id,
        attachment_heading,
        file_name,
        b64_encoded_attachment,
    ):
        """
        Create a ticket and updates it with an attachment.
        Returns either the ticket ID of the created ticket, or raises an exception.

        Attachment needs to be base64 encoded:
        # import base64
        # data = open("sample.txt", "r").read()
        # encoded = base64.b64encode(data.encode('utf-8'))
        # add_ticket_attachment(self, ticket_id, encoded)
        """

        ticket_id = self.api.create_ticket(
            title, description, status, priority, company_id, queue_id
        )

        self.api.add_ticket_attachment(
            ticket_id, attachment_heading, file_name, b64_encoded_attachment
        )

        return ticket_id